﻿Feature: Linking Work Items and other artifacts using tags

Synchronizing special work-item tags (e.g. @story:123) to work item links in test cases

Rule: Establish work item links based on scenario tags

@tc:155
Scenario: A work item link tag added to the scenario
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                           | value |
		| synchronization/links[]/tagPrefix | bug   |
	And a @bug tag has been added to an existing synchronized scenario as:
		"""
		@tc:[id-of-test-case] @bug:[id-of-bug]
		Scenario: Sample scenario
			When I do something
		"""
	When the local repository is synchronized with push
	Then the Test Case should be linked to the Bug

@tc:156
Scenario: A work item link tag added to the feature
	Given there is a remote server project with a Product Backlog Item
	And there is an existing synchronized feature file that has been changed to:
		"""
		@wi:[id-of-pbi]
		Feature: Sample feature

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                            | value |
		| synchronization/links[0]/tagPrefix | wi    |
	When the local repository is synchronized with push
	Then the Test Case should be linked to the Product Backlog Item

Rule: Links created by SpecSync are removed when the link tag is removed, but existing links or links that are manually added are preserved

@tc:157
Scenario: The work item tag is removed from the scenario
	Given there is a remote server project with a Product Backlog Item
	And the synchronizer is configured as
		| setting                            | value |
		| synchronization/links[0]/tagPrefix | wi    |
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case] @wi:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	When the scenario is updated and synchronized as:
		"""
		@tc:[id-of-test-case]
		Scenario: Updated sample scenario
			When I do something new
		"""
	Then the Test Case should not be linked to the Product Backlog Item

@tc:574 
@notSupported:JIRA.DataCenter.ZephyrScale @notSupported:JIRA.DataCenter.JiraTcm @notSupported:JIRA.Cloud.JiraTcm
Scenario: The manually linked work items are not removed
	Given there is a remote server project with a Product Backlog Item
	And there is a Bug in the project
	And the synchronizer is configured as
		| setting                            | value |
		| synchronization/links[0]/tagPrefix | wi    |
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case] @wi:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	And the Test Case has got the Bug linked from the remote server
	When the scenario is updated and synchronized as:
		"""
		@tc:[id-of-test-case]
		Scenario: Updated sample scenario
			When I do something new
		"""
	Then the Test Case should be still linked to the Bug

@tc:575
Scenario: Existing link synchronization is overtaken once linked with a tag
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                            | value |
		| synchronization/links[0]/tagPrefix | wi    |
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the Test Case has got the Bug linked from the remote server
	And the scenario has been updated and synchronized as:
		"""
		@tc:[id-of-test-case] @wi:[id-of-bug]
		Scenario: Sample scenario updated with the link
			When I do something
		"""
	When the scenario is updated and synchronized as:
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario removed link tag
			When I do something
		"""
	Then the Test Case should not be linked to the Bug


Rule: Adding a work item link should trigger an update

@tc:159
Scenario: Only a work item link tag is added
	Given there is a remote server project with a Product Backlog Item
	And there is an existing synchronized scenario
		"""
		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                            | value |
		| synchronization/links[0]/tagPrefix | wi    |
	When the scenario is updated to:
		"""
		@tc:[id-of-test-case] @wi:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	And the local repository is synchronized with push
	Then the Test Case should be linked to the Product Backlog Item

Rule: Multiple prefixes can be linked

@tc:158
Scenario: A product backlog link and a bug link tag is added to the scenario
	Given there is a remote server project
	And there is a Product Backlog Item in the project
	And there is a Bug in the project
	And there is an existing synchronized scenario that has been changed to:
		"""
		@tc:[id-of-test-case] @pbi:[id-of-pbi] @bug:[id-of-bug]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                            | value |
		| synchronization/links[0]/tagPrefix | pbi   |
		| synchronization/links[1]/tagPrefix | bug   |
	When the local repository is synchronized with push
	Then the Test Case should be linked to the Product Backlog Item
	And the Test Case should be linked to the Bug

Rule: Link type can be specified

@tc:160
Scenario: "Tests" link is created by default
	Given there is a remote server project with a Product Backlog Item
	And there is an existing synchronized scenario that has been changed to:
		"""
		@tc:[id-of-test-case] @wi:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                            | value |
		| synchronization/links[0]/tagPrefix | wi    |
	When the local repository is synchronized with push
	Then the Test Case should be linked to the Product Backlog Item as 'Tests'

@tc:161 
@notSupported:JIRA.DataCenter.ZephyrScale 
Scenario: The work item is linked as "Parent"
	Given there is a remote server project with a Product Backlog Item
	And there is an existing synchronized scenario that has been changed to:
		"""
		@tc:[id-of-test-case] @wi:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                               | value  |
		| synchronization/links[0]/tagPrefix    | wi     |
		| synchronization/links[0]/relationship | [parent-reference-type] |
	When the local repository is synchronized with push
	Then the Test Case should be linked to the Product Backlog Item as '[parent-reference-type]'

@tc:162 
@notSupported:JIRA.DataCenter.ZephyrScale 
Scenario: The link type has changed
	Given there is a remote server project with a Product Backlog Item
	And there is an existing synchronized scenario that has been changed to:
		"""
		@tc:[id-of-test-case] @wi:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                               | value |
		| synchronization/links[0]/tagPrefix    | wi    |
		| synchronization/links[0]/relationship | [tests-reference-type] |
	When the local repository is synchronized with push
	Given the synchronizer is configured as
		| setting                               | value  |
		| synchronization/links[0]/tagPrefix    | wi     |
		| synchronization/links[0]/relationship | [parent-reference-type] |
	And the synchronizer is configured to force updates
	When the local repository is synchronized with push
	Then the Test Case should be linked to the Product Backlog Item as '[parent-reference-type]'

Rule: Pull Requests can be linked

@tc:390 @adoSpecific
Scenario: A Pull Request link tag added to the scenario
	Given there is an Azure DevOps project with a Pull Request
	And there is an existing synchronized scenario that has been changed to:
		"""
		@tc:[id-of-test-case] @pr:[existing-pull-request]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                               | value        |
		| synchronization/links[0]/tagPrefix    | pr           |
		| synchronization/links[0]/relationship | Pull Request |
	When the local repository is synchronized with push
	Then the test case should be linked to the Pull Request

Rule: GitHub Pull Requests can be linked

@tc:825
@adoSpecific
@notSupported:tfs2017 @notSupported:tfs2018 
@notTested:ado2019 @notTested:ado2020 @notTested:ado2022 
Scenario Outline: A GitHub Pull Request link tag added to the scenario
	In ADO 2019 & ADO 2020, this feature is only supported for GitHub Enterprise Server (not for GutHub.com)
	See https://learn.microsoft.com/en-us/azure/devops/boards/github/troubleshoot-github-connection?view=azure-devops-2022
	Given there is an Azure DevOps project with a Pull Request
	And there is an existing synchronized scenario that has been changed to:
		"""
		@tc:[id-of-test-case] <PR tag>
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                               | value               |
		| synchronization/links[0]/tagPrefix    | pr                  |
		| synchronization/links[0]/relationship | GitHub Pull Request |
		| synchronization/links[0]/linkTemplate | <link template>     |
	When the local repository is synchronized with push
	Then the Test Case should be linked to the GitHub Pull Request
Examples: 
	| description   | PR tag                                                    | link template                  |
	| Link with URL | @pr:[github-project-url]/pull/[id-of-github-pull-request] |                                |
	| Link with ID  | @pr:[id-of-github-pull-request]                           | [github-project-url]/pull/{id} |
